"""Implements the PackageRepository interface for the local apt repository"""

import apt_pkg
import logging
from PackageRepository import PackageRepository


class AptPackageRepository(PackageRepository):
    """Implements a PackageRepository using the local apt repository"""

    def __init__(self):
        self.ready = False

    def _initialize(self):
        """Perfome one time initialize of the apt_pkg"""
        if self.ready:
            return
        log_str = "Initializing apt_pkg"
        logging.getLogger(__name__).info(log_str)
        apt_pkg.init()
        self.ready = True

    def get_build_dependencies(self, package_name):
        """Returns a set containing the build dependencies for the named
        binary package"""
        self._initialize()
        log_str = "get_build_dependencies: %s" % (package_name)
        logging.getLogger(__name__).info(log_str)
        build_deps = set()
        source_records = apt_pkg.SourceRecords()
        while source_records.lookup(package_name):
            # TODO: handle multiple source records, for now, just assume
            # the first one has what we need.
            for dep in source_records.build_depends["Build-Depends"]:
                log_str = "\t\tConsidering build-dep %s" % (dep)
                logging.getLogger(__name__).info(log_str)
                build_deps.add(dep[0][0])
            break
        return build_deps

    def get_binaries(self, source_name):
        """Returns a set containing the binaries for the named source
        package"""
        self._initialize()
        log_str = "get_binaries: %s" % (source_name)
        logging.getLogger(__name__).info(log_str)
        binaries = set()
        source_records = apt_pkg.SourceRecords()
        while source_records.lookup(source_name):
            # TODO: handle multiple source records, for now, just assume
            # the first one has what we need.
            binaries = binaries.union(set(source_records.binaries))
            break
        return binaries
