import ConfigParser
import logging
import sys

REQUIRED_SECTIONS = ["server",
                     "ci",
                     "al",
                     "pt"]
TEMPLATE_OPTIONS = ["build_template",
                    "check_template",
                    "dput_template",
                    "build_suffix",
                    "check_suffix",
                    "dput_suffix"]
REQUIRED_OPTIONS = {"server": ["hostname"],
                    "ci": ["template", "suffix"],
                    "al": ["template", "suffix"],
                    "pt": TEMPLATE_OPTIONS}


class ConfigError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class JenkinsConfig():
    """Manages configuration of the Jenkins server."""

    def __init__(self, file_name):
        self.config_file_name = file_name
        self.hostname = None
        self.username = None
        self.password = None

        self.config = ConfigParser.SafeConfigParser()
        self.config.read([file_name])

        # Perform error checking
        for section in REQUIRED_SECTIONS:
            if section not in self.config.sections():
                raise ConfigError("%s: Missing [%s] section" %
                                  (self.config_file_name, section))

        for key in REQUIRED_OPTIONS:
            opt_list = REQUIRED_OPTIONS[key]
            for opt in opt_list:
                if not self.config.has_option(key, opt):
                    raise ConfigError(("%s: Missing option [%s], " +
                                       "in section [%s]") %
                                      (self.config_file_name, opt, key))

        # Store the data
        self.hostname = self.config.get("server", "hostname")
        if self.config.has_option("server", "username"):
            self.username = self.config.get("server", "username")
        if self.config.has_option("server", "password"):
            self.password = self.config.get("server", "password")

        # TODO: Do something smarter with the parameter names.
        self.ci_template = self.config.get("ci", "template")
        self.ci_suffix = self.config.get("ci", "suffix")
        self.al_template = self.config.get("al", "template")
        self.al_suffix = self.config.get("al", "suffix")
        self.pt_build_template = self.config.get("pt", "build_template")
        self.pt_build_suffix = self.config.get("pt", "build_suffix")
        self.pt_check_template = self.config.get("pt", "check_template")
        self.pt_check_suffix = self.config.get("pt", "check_suffix")
        self.pt_dput_template = self.config.get("pt", "dput_template")
        self.pt_dput_suffix = self.config.get("pt", "dput_suffix")

    def __str__(self):
        return repr("[server: %s] -> [ci: %s, al: %s, pt: %s, %s]" %
                    (self.hostname, self.ci_template,
                     self.al_template,
                     self.pt_check_template,
                     self.pt_build_template))


def main(file_name):
    config = JenkinsConfig(file_name)
    print(config)


if __name__ == "__main__":
    main(sys.argv[1])
