#
# Builds Dependencies


import logging
import pprint
import networkx
import project as prj


class PackageDependencies():
    """Provides the build dependency connections between projects"""

    def __init__(self, package_repo, graph):
        self.package_repo = package_repo
        self.graph = graph

    def recordProjectDependencies(self, project):
        """Called per each node to fill in upstream and downstream build
        dependencies"""
        # target_branch is used to determine if this project is built by MBS
        if project.parameters["target_branch"] is None:
            return
        for node in self.graph.successors(project):
            if node.parameters["target_branch"] is not None:
                project.downStream.insert(0, node.name)
        for node in self.graph.predecessors(project):
            if node.parameters["target_branch"] is not None:
                project.upStream.insert(0, node.parameters["target_branch"])

    def add_to_dependency_graph(self, project):
        log_str = "Adding [%s] to graph: %s" % (project.name,
                                                project.build_dependencies)
        logging.getLogger(__name__).info(log_str)
        self.graph.add_node(project, style="filled", color="green",
                            fillcolor="green")
        update = False
        for node in self.graph.nodes():
            log_str = "\t\tnode: %s" % (node.name)
            logging.getLogger(__name__).info(log_str)
            log_str = "\t\t\tpackages: %s" % (node.packages)
            logging.getLogger(__name__).info(log_str)
            if project.build_dependencies.intersection(node.packages):
                # A build_dependency for project is satisfied by node
                # Node is upstream of project
                if not self.graph.has_edge(node, project):
                    update = True
                    self.graph.add_edge(node, project)
                    log_str = ("Matched: %s is upstream of %s" %
                               (node.name, project.name))
                    logging.getLogger(__name__).info(log_str)

            if project.packages.intersection(node.build_dependencies):
                # A build_dependency for node is satisfied by project
                # Node is downstream of project
                if not self.graph.has_edge(project, node):
                    update = True
                    self.graph.add_edge(project, node)
                    log_str = ("Matched: %s is downstream of %s" %
                               (node.name, project.name))
                    logging.getLogger(__name__).info(log_str)

    def generateDeps(self, project):
        """Identifies the build dependencies for a given project"""
        build_deps = set()
        for package in project.packages:
            log_str = "\tConsidering package: %s under %s" % (package,
                                                              project.name)
            logging.getLogger(__name__).info(log_str)
            build_deps = build_deps.union(
                    self.package_repo.get_build_dependencies(package))
        project.build_dependencies = build_deps.union(
                set(project.manualDependencies))
