import logging
import pprint
import sys


class Project:
    """Project encapsulates all project specific attributes"""

    FORMAT_PATTERN = ("'%s:\\ntarget(%s)\\npackaging(%s)\\n" +
                      "pbuilder_hooks(%s)\\nupstream(%s)\\ndownstream(%s)'")
    NEW = 1
    UPDATED = 2
    PARAMETER_LIST = ["pbuilder_hooks",
                      "target_branch",
                      "packaging_branch",
                     ]

    def __init__(self, name, index):
        self.name = name

        # Initialize default parameter values
        self.parameters = {}
        for p in self.PARAMETER_LIST:
            self.parameters[p] = None

        self.downStream = []
        self.upStream = []
        self.packages = set()
        self.build_dependencies = set()
        self.index = index

    def __hash__(self):
        return hash(self.name)

    def __eq__(self, other):
        return self.name == other.name

    def __str__(self):
        return Project.FORMAT_PATTERN % (self.name,
                                         self.parameters["target_branch"],
                                         self.parameters["packaging_branch"],
                                         self.parameters["pbuilder_hooks"],
                                         self.upStream,
                                         self.downStream)
