#/usr/bin/env python

#from mock import MagicMock, patch
import unittest
import os.path
from JenkinsConfig import *

DATA_PATH = "tests/data"
JENKINS_CONFIG_NAME = "jenkins_tests.cfg"


class JenkinsConfigTestCase(unittest.TestCase):
    """Unit tests for class JenkinsConfig"""

    def setUp(self):
        """Setup for all test cases"""
        self.jenkins_config = JenkinsConfig(os.path.join(DATA_PATH,
                                                         JENKINS_CONFIG_NAME))

    def tearDown(self):
        """Tear down for all test cases"""
        pass

    def testRequiredServerOptions(self):
        """Verify extraction of required server options"""
        self.assertEqual(self.jenkins_config.hostname,
                         "http://localhost:8081/")

    def testOptionalServerOptions(self):
        """Verify extraction of optional server options"""
        self.assertEqual(self.jenkins_config.username, "jenkins-user")
        self.assertEqual(self.jenkins_config.password, "jenkins-password")

    def testRequiredCiOptions(self):
        """Verify extraction of required ci options"""
        self.assertEqual(self.jenkins_config.ci_template, "template-mbs-ci")
        self.assertEqual(self.jenkins_config.ci_suffix, "mbs-ci")

    def testRequiredAlOptions(self):
        """Verify extraction of required autolanding options"""
        self.assertEqual(self.jenkins_config.al_template, "template-mbs-al")
        self.assertEqual(self.jenkins_config.al_suffix, "mbs-al")

    def testRequiredPtOptions(self):
        """Verify extraction of required autolanding options"""
        self.assertEqual(self.jenkins_config.pt_build_template,
                         "template-build-mbs-pt")
        self.assertEqual(self.jenkins_config.pt_build_suffix, "build-mbs-pt")
        self.assertEqual(self.jenkins_config.pt_check_template,
                         "template-check-mbs-pt")
        self.assertEqual(self.jenkins_config.pt_check_suffix, "check-mbs-pt")

    def testString(self):
        self.assertIn("[server: http://localhost:8081/]",
                      "%s" % self.jenkins_config)
        self.assertIn("ci: template-mbs-ci",
                      "%s" % self.jenkins_config)
        self.assertIn("al: template-mbs-al",
                      "%s" % self.jenkins_config)
        self.assertIn("pt: template-check-mbs-pt, template-build-mbs-pt",
                      "%s" % self.jenkins_config)


class JenkinsConfigErrorsTestCase(unittest.TestCase):
    """Unit tests for JenkinsConfig exceptions"""

    def testMissingServerSection(self):
        """Verifies ConfigError is raised when missing the server section"""
        config_file = "jenkins_tests_missing_server_section.cfg"
        with self.assertRaises(ConfigError) as cm:
            JenkinsConfig(os.path.join(DATA_PATH,
                                        config_file))
        self.assertIn(config_file, "%s" % (cm.exception))
        self.assertIn("Missing [server] section", "%s" % (cm.exception))

    def testMissingServerOption(self):
        """Verifies ConfigError is raised when missing a required server
        option"""
        config_file = "jenkins_tests_missing_required_option.cfg"
        with self.assertRaises(ConfigError) as cm:
            JenkinsConfig(os.path.join(DATA_PATH,
                                        config_file))
        self.assertIn(config_file, "%s" % (cm.exception))
        self.assertIn("Missing option [hostname]", "%s" % (cm.exception))
        self.assertIn("in section [server]", "%s" % (cm.exception))

    def testMissingCiSection(self):
        """Verifies ConfigError is raised when missing the ci section"""
        config_file = "jenkins_tests_missing_ci_section.cfg"
        with self.assertRaises(ConfigError) as cm:
            JenkinsConfig(os.path.join(DATA_PATH,
                                        config_file))
        self.assertIn(config_file, "%s" % (cm.exception))
        self.assertIn("Missing [ci] section", "%s" % (cm.exception))
